import asyncio
import inspect
import shlex
from collections.abc import Callable, Coroutine
from typing import Annotated, Any

from fastmcp import FastMCP
from pydantic import BaseModel, Field

from unpage.plugins.base import Plugin
from unpage.plugins.mixins import McpServerMixin


class ShellCommand(BaseModel):
    handle: str
    description: str
    command: str
    args: dict[str, str] = Field(default_factory=dict)


class ShellPluginSettings(BaseModel):
    commands: list[ShellCommand] = Field(default_factory=list)


class ShellPlugin(Plugin, McpServerMixin):
    shell_settings: ShellPluginSettings = Field(default_factory=ShellPluginSettings)

    def init_plugin(self) -> None:
        self.shell_settings = ShellPluginSettings(**self._settings)

    def get_mcp_server(self) -> FastMCP[Any]:
        mcp = super().get_mcp_server()
        for command_config in self.shell_settings.commands:
            mcp.tool(self._build_tool_function(command_config))
        return mcp

    def _build_tool_function(
        self, command_config: ShellCommand
    ) -> Callable[..., Coroutine[Any, Any, str]]:
        """Create a dynamic MCP tool for a shell command."""

        # Create an anonymous function that executes the command.
        async def _tool_function(*args: str, **kwargs: str) -> str:
            escaped_args = {k: shlex.quote(v) for k, v in kwargs.items()}
            rendered_command = command_config.command.format(**escaped_args)

            process = await asyncio.create_subprocess_shell(
                rendered_command,
                # Combine stdout and stderr into a single stream
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.STDOUT,
            )
            stdout, _ = await process.communicate()
            result = stdout.decode()

            if process.returncode != 0:
                return f"Command returned non-zero code {process.returncode}: {result}"

            return result

        # Set the function metadata to match the command config.
        _tool_function.__name__ = command_config.handle
        _tool_function.__doc__ = command_config.description

        # Set the function signature so that FastMCP can inspect it.
        _tool_function.__signature__ = inspect.Signature(  # type: ignore
            [
                inspect.Parameter(
                    arg_name,
                    inspect.Parameter.POSITIONAL_OR_KEYWORD,
                    annotation=Annotated[str, Field(description=arg_description)],
                )
                for arg_name, arg_description in command_config.args.items()
            ]
        )

        # Set the function annotations to match the signature.
        _tool_function.__annotations__ = {
            **{
                arg_name: Annotated[str, Field(description=arg_description)]
                for arg_name, arg_description in command_config.args.items()
            },
            "return": str,
        }

        return _tool_function



from rich import print

from unpage.agent.utils import get_agents
from unpage.cli.agent._app import agent_app
from unpage.config import manager
from unpage.telemetry import client as telemetry
from unpage.telemetry import prepare_profile_for_telemetry


@agent_app.command
async def list() -> None:
    """List the available agents."""
    await telemetry.send_event(
        {
            "command": "agent list",
            **prepare_profile_for_telemetry(manager.get_active_profile()),
        }
    )
    print("Available agents:")
    for agent in sorted(get_agents()):
        print(f"* {agent}")



import os
from typing import Any, Literal

import questionary
from fastmcp.mcp_config import MCPServerTypes, RemoteMCPServer, StdioMCPServer
from questionary import select

from unpage.config import PluginSettings
from unpage.plugins.mcp.plugin import McpProxyPlugin


class SentryPlugin(McpProxyPlugin):
    """Sentry plugin."""

    name = "sentry"

    def __init__(
        self,
        *args: Any,
        mode: Literal["remote", "local"] | None = None,
        sentry_token: str | None = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(*args, **kwargs)
        self.mode = mode or os.getenv("SENTRY_MODE", "remote")
        self.sentry_token = sentry_token or os.getenv("SENTRY_TOKEN")

        if self.mode == "local" and not self.sentry_token:
            raise ValueError("sentry_token is required when mode is 'local'")

    async def interactive_configure(self) -> PluginSettings:
        mode = await select(
            "The Sentry plugin uses the Sentry MCP Server for some of its features. Do you want to run the MCP server locally, or use Sentry's hosted MCP server?",
            choices=["local", "remote"],
            default=self.mode,
        ).unsafe_ask_async()

        sentry_token = (
            await questionary.password(
                "Enter your Sentry API token",
                default=self.sentry_token or "",
            ).unsafe_ask_async()
            if mode == "local"
            else None
        )

        return {
            "mode": mode,
            "sentry_token": sentry_token,
        }

    async def validate_plugin_config(self) -> None:
        """Validate the plugin config."""
        try:
            await super().validate_plugin_config()
            await self.call_tool("sentry_whoami")
        except Exception as ex:
            raise ValueError(f"Error validating {self.name!r}: {ex}") from ex

    def get_mcp_server_settings(self) -> MCPServerTypes:
        if self.mode == "local":
            return self._get_local_mcp_server_settings()
        elif self.mode == "remote":
            return self._get_remote_mcp_server_settings()
        else:
            raise ValueError(f"Invalid mode {self.mode!r} (must be 'local' or 'remote')")

    def _get_local_mcp_server_settings(self) -> StdioMCPServer:
        return StdioMCPServer(
            command="npx",
            args=["@sentry/mcp-server"],
            env={
                "SENTRY_ACCESS_TOKEN": self.sentry_token,
            },
        )

    def _get_remote_mcp_server_settings(self) -> RemoteMCPServer:
        return RemoteMCPServer(
            url="https://mcp.sentry.dev/mcp",
            auth="oauth",
        )



import os
from typing import Any, Literal

import questionary
from fastmcp.mcp_config import MCPServerTypes, RemoteMCPServer, StdioMCPServer
from questionary import select

from unpage.config import PluginSettings
from unpage.plugins.mcp.plugin import McpProxyPlugin


class GitHubPlugin(McpProxyPlugin):
    """GitHub plugin."""

    name = "github"

    def __init__(
        self,
        *args: Any,
        github_token: str | None = None,
        mode: Literal["remote", "local"] = "local",
        read_only: bool = True,
        **kwargs: Any,
    ) -> None:
        super().__init__(*args, **kwargs)
        self.github_token = github_token or os.getenv("GITHUB_PERSONAL_ACCESS_TOKEN")
        self.mode = mode or os.getenv("GITHUB_MODE", "remote")
        self.read_only = read_only

        if not self.github_token:
            raise ValueError("github_token is required")

    async def interactive_configure(self) -> PluginSettings:
        return {
            "mode": await select(
                "The GitHub plugin uses the GitHub MCP Server for some of its features. Do you want to use the local or remote MCP server?",
                choices=["local", "remote"],
                default=self.mode,
            ).unsafe_ask_async(),
            "github_token": await questionary.password(
                "Enter your GitHub Personal Access Token",
                default=self.github_token or "",
            ).unsafe_ask_async(),
        }

    async def validate_plugin_config(self) -> None:
        """Validate the plugin config."""
        try:
            await super().validate_plugin_config()
            await self.call_tool("github_get_me")
        except Exception as ex:
            raise ValueError(f"Error validating {self.name!r}: {ex}") from ex

    def get_mcp_server_settings(self) -> MCPServerTypes:
        if self.mode == "local":
            return self._get_local_mcp_server_settings()
        elif self.mode == "remote":
            return self._get_remote_mcp_server_settings()
        else:
            raise ValueError(f"Invalid mode {self.mode!r} (must be 'local' or 'remote')")

    def _get_remote_mcp_server_settings(self) -> RemoteMCPServer:
        return RemoteMCPServer(
            transport="http",
            url="https://api.githubcopilot.com/mcp/",
            headers={
                "X-MCP-Readonly": "1" if self.read_only else "0",
            },
            auth=self.github_token or "oauth",
        )

    def _get_local_mcp_server_settings(self) -> StdioMCPServer:
        return StdioMCPServer(
            command="docker",
            args=[
                "run",
                "--rm",
                "-i",
                "-e",
                "GITHUB_PERSONAL_ACCESS_TOKEN",
                "-e",
                "GITHUB_READ-ONLY",
                "ghcr.io/github/github-mcp-server",
            ],
            env={
                "GITHUB_PERSONAL_ACCESS_TOKEN": self.github_token,
                "GITHUB_READ-ONLY": "1" if self.read_only else "0",
            },
        )



import sys

import rich

from unpage.agent.utils import delete_agent
from unpage.cli.agent._app import agent_app
from unpage.config import manager
from unpage.telemetry import client as telemetry
from unpage.telemetry import hash_value, prepare_profile_for_telemetry


@agent_app.command
async def delete(agent_name: str) -> None:
    """Delete an agent.

    Parameters
    ----------
    agent_name
        The name of the agent to delete
    """
    await telemetry.send_event(
        {
            "command": "agent delete",
            "agent_name_sha256": hash_value(agent_name),
            **prepare_profile_for_telemetry(manager.get_active_profile()),
        }
    )
    try:
        delete_agent(agent_name)
        rich.print(f"[green]Agent '{agent_name}' deleted successfully[/green]")
    except FileNotFoundError:
        rich.print(f"[red]Agent '{agent_name}' does not exist[/red]")
        sys.exit(1)



from pydantic import AwareDatetime

from unpage.models import LogLine


class HasLogs:
    """A mixin for nodes that support retrieving logs."""

    async def get_logs(
        self,
        time_range_start: AwareDatetime,
        time_range_end: AwareDatetime,
    ) -> list[LogLine]:
        """Retrieve the node's logs for a given time range."""
        raise NotImplementedError
